/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* 
* Copyright (C) 2017-2019 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/******************************************************************************
* File Name   : mtr_calc_encd_foc.c
* Description : The processing of a calculation layer
******************************************************************************/
/******************************************************************************
* History : 29.05.2017  Ver.1.00
******************************************************************************/

/******************************************************************************
* Includes <System Includes> , "Project Includes"
******************************************************************************/
#include "mtr_common.h"
#include "r_dsp.h"
#include "motor_parameter.h"
#include "mtr_calc_encd_foc.h"

/******************************************************************************
* Exported global variables
******************************************************************************/
extern volatile int16_t    g_s2_foc_encd_status;                        /* encoder vector control status */

extern volatile int16_t    g_s2_foc_mech_count;                         /* count value(mechanical) */
extern volatile int16_t    g_s2_foc_elec_count;                         /* count value(electrical) */

/******************************************************************************
* Private global variables and functions
******************************************************************************/
static uint16_t   g_u2_foc_encd_count;                                  /* count value */

static int16_t    g_s2_foc_calc_count;                                  /* count value */

static float32_t  g_f4_foc_lpf_zl;                                      /* delay element of the LPF */

static void mtr_calc_LPF(float32_t*);

/******************************************************************************
* Function Name : mtr_calc_LPF
* Description   : This function calculates the LPF.
* Arguments     : float32_t *p_f4_io
* Return Value  : none
******************************************************************************/
static void mtr_calc_LPF(float32_t *p_f4_io)
{
    *p_f4_io += (MTR_COEF_LPF1 * g_f4_foc_lpf_zl);

    g_f4_foc_lpf_zl = *p_f4_io;

    *p_f4_io *= MTR_COEF_LPF2;
}

/******************************************************************************
* Function Name : mtr_calc_InitVariables
* Description   : This function initializes private global variables.
* Arguments     : none
* Return Value  : none
******************************************************************************/
void mtr_calc_InitVariables(void)
{
    g_u2_foc_encd_count = 0;

    g_s2_foc_calc_count = 1;

    g_f4_foc_lpf_zl = 0.0f;
}

/******************************************************************************
* Function Name : mtr_calc_PID
* Description   : This function calculates the PID.
* Arguments     : int16_t  s2_in
*                 int16_t *p_s2_out
*                 MTR_PID_PRM prm
* Return Value  : none
******************************************************************************/
void mtr_calc_PID(int16_t s2_in, int16_t *p_s2_out, MTR_PID_PRM prm)
{
    float32_t f4_temp, f4_temp_p, f4_temp_i, f4_temp_zi, f4_temp_d;

    f4_temp_p = (float32_t)s2_in;

    f4_temp = f4_temp_p * prm.f4_ti;

    f4_temp_i = f4_temp + *(prm.f4_zi);

    f4_temp_zi = f4_temp + f4_temp_i;

    if (f4_temp_zi > MTR_PID_LIMIT_ZI)
    {
        f4_temp_zi = MTR_PID_LIMIT_ZI;
    }
    else if (f4_temp_zi < (-MTR_PID_LIMIT_ZI))
    {
        f4_temp_zi = (-MTR_PID_LIMIT_ZI);
    }
    else
    {
        /* Do Nothing */
    }

    *(prm.f4_zi) = f4_temp_zi;

    f4_temp = f4_temp_p * prm.f4_td;

    f4_temp_d = f4_temp - *(prm.f4_zd);

    *(prm.f4_zd) = f4_temp;

    f4_temp = (prm.f4_kp * f4_temp_p + prm.f4_ki * f4_temp_i + prm.f4_kd * f4_temp_d) * prm.f4_kt;

    *p_s2_out = (int16_t)(f4_temp);
}

/******************************************************************************
* Function Name : mtr_calc_SRL
* Description   : This function calculates the SRL.
* Arguments     :  int16_t  s2_in
*                  int16_t *p_s2_zs
*                 uint16_t  s2_limit
* Return Value  :  int16_t  s2_out
******************************************************************************/
int16_t mtr_calc_SRL(int16_t s2_in, int16_t *p_s2_zs, uint16_t u2_limit)
{
    int16_t s2_temp;

    s2_temp = s2_in;

    s2_temp -= *p_s2_zs;

    s2_temp = R_FIX_limit_int16(s2_temp, u2_limit);

    s2_temp += *p_s2_zs;

    *p_s2_zs = s2_temp;

    return (s2_temp);
}

/******************************************************************************
* Function Name : mtr_calc_EncoderInput
* Description   : This function calculates the input from the encoder.
* Arguments     : int16_t *p_s2_theta_elec electrical angle
*                 int16_t *p_s2_theta_mech mechanical angle
*                 int16_t *p_s2_omega_mech mechanical speed
* Return Value  : none
******************************************************************************/
void mtr_calc_EncoderInput(int16_t *p_s2_theta_elec, int16_t *p_s2_theta_mech, int16_t *p_s2_omega_mech)
{
    int16_t s2_delta;                                                   /* delta count value */

    {
        uint16_t u2_count;

        u2_count = (uint16_t)TRG;

        if (TRGOVF)                                                     /* check overflow flag */
        {
            TRGOVF = 0;                                                 /* clear overflow flag */

            s2_delta = USHRT_MAX - (int16_t)(g_u2_foc_encd_count - u2_count) + 1;
        }
        else if (TRGUDF)                                                /* check underflow flag */
        {
            TRGUDF = 0;                                                 /* clear underflow flag */

            s2_delta = (int16_t)(u2_count - g_u2_foc_encd_count) - USHRT_MAX - 1;
        }
        else
        {
            s2_delta = (int16_t)(u2_count - g_u2_foc_encd_count);
        }

        g_u2_foc_encd_count = u2_count;                                 /* update count value */
    }

    {
        float32_t f4_omega;

        f4_omega = MTR_COEF_ENCD_OMEGA * (float32_t)s2_delta;           /* calculate mechanical speed */

        mtr_calc_LPF(&f4_omega);

        *p_s2_omega_mech = (int16_t)(f4_omega * 16.0f);                 /* convert scale(Q0 -> Q4) */
    }

    g_s2_foc_mech_count += s2_delta;

    if (g_s2_foc_mech_count < 0)                                        /* Fail Safe */
    {
        g_s2_foc_mech_count += MTR_ENCD_CPR_MECH;
    }
    else if (g_s2_foc_mech_count >= MTR_ENCD_CPR_MECH)
    {
        g_s2_foc_mech_count -= MTR_ENCD_CPR_MECH;
    }
    else
    {
        /* Do Nothing */
    }

    switch (g_s2_foc_encd_status)
    {
        case MTR_ENCD_STATE_ALIGN:                                      /* calculate the theta from the counter(alignment) */
            g_s2_foc_elec_count = g_s2_foc_calc_count;

            if (g_s2_foc_elec_count == 0)                               /* calculate the offset and change the calculation of theta */
            {
                g_s2_foc_mech_count = 0;

                g_s2_foc_encd_status = MTR_ENCD_STATE_INIT;
            }
        break;

        case MTR_ENCD_STATE_INIT:                                       /* calculate the theta from the counter(initial position) */
            g_s2_foc_elec_count = g_s2_foc_calc_count;

            if (g_s2_foc_elec_count == 0)                               /* move to avoid the neighborhood of 0 */
            {
                g_s2_foc_encd_status = MTR_ENCD_STATE_DEFAULT;
            }
        break;

        case MTR_ENCD_STATE_CURRENT:                                    /* calculate the theta from the timer RG(current control) */
        case MTR_ENCD_STATE_SPEED:                                      /* calculate the theta from the timer RG(speed control) */
        case MTR_ENCD_STATE_POSITION:                                   /* calculate the theta from the timer RG(position control) */
            g_s2_foc_elec_count = (g_s2_foc_mech_count * MTR_ENCD_POLE_PAIRS) % MTR_ENCD_CPR_MECH;
        break;

        default: 
        break;
    }

    /* calculate mechanical angle */
    *p_s2_theta_mech = (int16_t)(MTR_COEF_ENCD_THETA * (float32_t)g_s2_foc_mech_count);

    /* calculate electrical angle */
    *p_s2_theta_elec = (int16_t)(MTR_COEF_ENCD_THETA * (float32_t)g_s2_foc_elec_count);

    g_s2_foc_calc_count++;                                              /* update count value */

    if (g_s2_foc_calc_count == MTR_ENCD_CPR_MECH)
    {
        g_s2_foc_calc_count = 0;
    }
}

